#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import atexit
import logging
logger = logging.getLogger('PyAMS (content.es)')

import os.path
import sys

# import interfaces
from pyams_content_es.interfaces import INDEXER_HANDLER_KEY, INDEXER_STARTER_KEY, INDEXER_AUTH_KEY, INDEXER_CLIENTS_KEY
from pyramid.interfaces import IApplicationCreated

# import packages
from pyams_content_es.process import ContentIndexerProcess, ContentIndexerMessageHandler
from pyams_utils.protocol.tcp import is_port_in_use
from pyams_utils.registry import get_global_registry
from pyams_zmq.process import process_exit_func
from pyramid.events import subscriber
from pyramid.settings import asbool


def include_package(config):
    """Pyramid include"""

    # add translations
    config.add_translation_dirs('pyams_content_es:locales')

    # load registry components
    try:
        import pyams_zmi
    except ImportError:
        config.scan(ignore='pyams_content_es.zmi')
    else:
        config.scan()


@subscriber(IApplicationCreated)
def handle_new_application(event):
    """Start indexer process when application created"""

    # check for PyAMS command line script
    cmdline = os.path.split(sys.argv[0])[-1]
    if cmdline.startswith('pyams_'):
        return

    registry = get_global_registry()
    settings = registry.settings
    start_handler = asbool(settings.get(INDEXER_STARTER_KEY, False))
    if start_handler:
        # check if port is available
        handler_address = settings.get(INDEXER_HANDLER_KEY, '127.0.0.1:5557')
        hostname, port = handler_address.split(':')
        if is_port_in_use(int(port), hostname):
            logger.info("Elasticsearch content indexer port already used, aborting...")
            return
        # create content indexer process
        process = None
        try:
            process = ContentIndexerProcess(handler_address,
                                            ContentIndexerMessageHandler,
                                            settings.get(INDEXER_AUTH_KEY),
                                            settings.get(INDEXER_CLIENTS_KEY),
                                            registry)
            logger.info('Starting Elasticsearch content indexer {0!r}...'.format(process))
            process.start()
            if process.is_alive():
                atexit.register(process_exit_func, process=process)
                logger.info('Started Elasticsearch content indexer with PID {0}...'.format(process.pid))
        finally:
            if process and not process.is_alive():
                process.terminate()
                process.join()
