#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import transaction

from pyramid_es import client_from_config
from pyams_content.shared.common import IWfSharedContent
from pyams_content_es.interfaces import IContentIndexerUtility, IDocumentIndexTarget, INDEXER_NAME
from pyams_content_es.utility import ContentIndexerUtility
from pyams_utils.container import find_objects_providing
from pyams_utils.interfaces.site import ISiteGenerations
from pyams_utils.registry import query_utility, set_local_registry, utility_config
from pyams_utils.site import check_required_utilities, site_factory


REQUIRED_UTILITIES = ((IContentIndexerUtility, '', ContentIndexerUtility, INDEXER_NAME),)


@utility_config(name='PyAMS content indexer', provides=ISiteGenerations)
class ContentIndexerGenerationsChecker(object):
    """Content indexer utility generations checker"""

    order = 110
    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)


def site_index(request, root=None, include=None, exclude=None, timeout=1.0):
    """Index all site contents in ElasticSearch"""
    registry = request.registry
    es_client = client_from_config(registry.settings, timeout=timeout)
    application = site_factory(request)
    if application is not None:
        try:
            set_local_registry(application.getSiteManager())
            indexer = query_utility(IContentIndexerUtility)
            if indexer is not None:
                if root:
                    for part in filter(bool, root.split('/')):
                        application = application[part]
                for index, document in enumerate(find_objects_providing(application,
                                                                        IDocumentIndexTarget)):
                    try:
                        if (include or exclude) and IWfSharedContent.providedBy(document):
                            if (include and document.content_type not in include) or \
                                    (exclude and document.content_type in exclude):
                                continue
                        print("Indexing: {0!r}".format(document))
                        es_client.index_object(document)
                        if not (index % 10):
                            transaction.commit()
                    except:
                        continue
        finally:
            set_local_registry(None)
        transaction.commit()
    return application
